﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

// including the M2Mqtt Library
using uPLibrary.Networking.M2Mqtt;
using uPLibrary.Networking.M2Mqtt.Messages;

// including the Serial Interface Library
using System.IO.Ports;


namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        MqttClient client;
        string clientId;

        SerialPort COMPort = null;

        // this code runs when the main window opens (start of the app)
        public MainWindow()
        {
            InitializeComponent();


            //string BrokerAddress = "test.mosquitto.org";
            string BrokerAddress = "broker.hivemq.com";

            client = new MqttClient(BrokerAddress);

            // register a callback-function (we have to implement, see below) which is called by the library when a message was received
            client.MqttMsgPublishReceived += client_MqttMsgPublishReceived;

            // use a unique id as client id, each time we start the application
            clientId = Guid.NewGuid().ToString();

            client.Connect(clientId);

            COMPortsRefresh();

        }


        // this code runs when the main window closes (end of the app)
        protected override void OnClosed(EventArgs e)
        {
            COMPort.Close();
            client.Disconnect();

            base.OnClosed(e);
            App.Current.Shutdown();
        }



        // this code runs when the button "Subscribe" is clicked
        private void btnSubscribe_Click(object sender, RoutedEventArgs e)
        {
            if (txtTopicSubscribe.Text != "")
            {
                // whole topic
                string Topic = "/ElektorMyJourneyIoT/" + txtTopicSubscribe.Text + "/test";

                // subscribe to the topic with QoS 2
                client.Subscribe(new string[] { Topic }, new byte[] { 2 });   // we need arrays as parameters because we can subscribe to different topics with one call
                txtReceived.Text = "";
            }
            else
            {
                System.Windows.MessageBox.Show("You have to enter a topic to subscribe!");
            }
        }


        // this code runs when the button "Publish" is clicked
        private void btnPublish_Click(object sender, RoutedEventArgs e)
        {
            if (txtTopicPublish.Text != "")
            {
                // whole topic
                string Topic = "/ElektorMyJourneyIoT/" + txtTopicPublish.Text + "/test";

                // publish a message with QoS 2
                client.Publish(Topic, Encoding.UTF8.GetBytes(txtPublish.Text), MqttMsgBase.QOS_LEVEL_EXACTLY_ONCE, true);
            }
            else
            {
                System.Windows.MessageBox.Show("You have to enter a topic to publish!");
            }
        }


        // this code runs when a message was received
        void client_MqttMsgPublishReceived(object sender, MqttMsgPublishEventArgs e)
        {
            string ReceivedMessage = Encoding.UTF8.GetString(e.Message);

            Dispatcher.Invoke(delegate {              // we need this construction because the receiving code in the library and the UI with textbox run on different threads
                txtReceived.Text = ReceivedMessage;

                if (COMPort != null)
                {
                    
                    COMPort.Write(Encoding.UTF8.GetString(e.Message) + "\r");
                    
                }

            });

        }


        // this code runs when user changes the COM-Port selection
        private void cboPorts_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (cboPorts.SelectedValue != null)
            { 
                if (COMPort == null)
                {
                    // Port with Portname in combobox and baudrate 9600
                    COMPort = new SerialPort((String)cboPorts.SelectedValue, 9600);
                    // DTR Pin Enable
                    COMPort.DtrEnable = true;
                }
                else
                {
                    COMPort.Close();
                    COMPort.PortName = (String)cboPorts.SelectedValue;
                }
                COMPort.Open();
            }
        }


        // this code runs when the button "Refresh" is clicked
        private void btnCOMPortsRefresh_Click(object sender, RoutedEventArgs e)
        {
            COMPortsRefresh();
        }


        // fill combobox with Names of available COM-Ports
        private void COMPortsRefresh()
        {
            String[] ports = SerialPort.GetPortNames();
            cboPorts.Items.Clear();
            foreach (String port in ports)
            {
                cboPorts.Items.Add(port);
            }
        }


    }
}
