/*
IOT Sensor Board Reference Application 1
for Pretzel-Board
Elektor / Jens Nickel
My Journey in the IoT
 */
 
#include <SoftwareSerial.h>

SoftwareSerial esp8266(11, 12);



// +++++++++++++++++++ Start of MQTT Library ++++++++++++++++++

// MQTT Library functions based on PubSubClient by Nick O'Leary: http://pubsubclient.knolleary.net/index.html
// MIT-License: https://opensource.org/licenses/mit-license.php


byte buffer[128];

byte MQTTCONNECT = 16;
byte MQTTPUBLISH = 48;


// Send MQTT-connection-request to MQTT-Server via TCP
int MQTTClient_Connect(String clientid) 
{
    int result = 0;

    // fill the buffer with data to be sent for connection request
    int BufferIndex = 2;  //  we start at index 2 to leave room for message type byte and remaininglength byte

    // for MQTT version 3.1.1 you always have to sent the following bytes
    byte d[7] = { 0, 4, 77, 81, 84, 84, 4};

    // fill the buffer with these bytes
    for (int j = 0 ;j < 7; j++) 
    {
        buffer[BufferIndex++] = d[j];
    }

    // next byte position is filled with flags, we only use the clean session flag
    buffer[BufferIndex++] = 0x02;

    // next two bytes are keep alive time
    buffer[BufferIndex++] = 0;
    buffer[BufferIndex++] = 180;

    // then write the client-ID in the buffer, with length bytes at the beginning
    BufferIndex = MQTTClient_WriteStringInBuffer(clientid, buffer, BufferIndex);

    // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
    result = MQTTClient_Send(MQTTCONNECT, buffer, BufferIndex - 2); 
  
    return result;  
}



// Send MQTT-Publish-message to MQTT-Server via TCP
// the byte array payload contains the payload data, plength is the amount of payload bytes
int MQTTClient_Publish(String topic, byte payload[], int plength) 
{
    // fill the buffer with data to be sent for publish message
    int BufferIndex = 2;  //  we start at index 2 to leave room for message type byte and remaininglength byte

    // write the topic in the buffer, with topic length bytes at the beginning and offset = 2
    BufferIndex = MQTTClient_WriteStringInBuffer(topic, buffer, BufferIndex);

    // then copy the payload bytes in the buffer
    for (int i = 0; i  < plength; i++)
    {
        buffer[BufferIndex++] = payload[i];
    }

    // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
    return MQTTClient_Send(MQTTPUBLISH, buffer, BufferIndex - 2);
}




// send out a MQTT message from client to server
// send the buffer via TCP/IP, put message type and remaining length byte at the beginning
// buf contains the bytes to be sent out, with 2 bytes left free at the beginning
// RemainingLengthBytes is the number of bytes, without the 2 bytes at the beginning
int MQTTClient_Send(byte MessageType, byte buf[], int RemainingLengthByte)
{   
    buf[0] = MessageType;        
    buf[1] = RemainingLengthByte & 255;
      
    // send out all bytes via TCP/IP
    int result = TCPClient_Send(buf, RemainingLengthByte + 2);

    return result;
}





// this function writes the bytes (ASCII) of a string in the buffer buf, with a given offset, and returns a new offset/index back
int MQTTClient_WriteStringInBuffer(String MString, byte buf[], int offset) 
{
  
    int Slength = MString.length();
    
    // copy the bytes of this array in the buffer, with the given offset, and leave 2 bytes free at the beginning
    for (int j = 0; j < Slength; j++)
    {
        buf[offset + 2 + j] = MString[j];
    }

    // fill the lenght of the string in the first 2 bytes (string length is lower than 128 in that case)
    buf[offset] = 0;
    buf[offset + 1] = Slength;

    // return new offset/index after the string
    return offset + 2 + Slength;
}




// +++++++++++++++++++ End of MQTT library ++++++++++++++++++++


       


// +++++++++++++++++++ Start of TCP Library +++++++++++++++++++

// as ESP8266 AT-command version



// Closes TCP/IP-connection
// returns 1 when ok, 0 when error

int TCPClient_Close()
{
    int Result = 0;

    
    // send AT-command to ESP8266 to close the connection
    esp8266.println("AT+CIPCLOSE");

    
    // waits until ESP8266 send back "OK" (success = true) or "ERROR" (success = false)
    boolean success = esp8266.findUntil("OK", "ERROR");
    
    if (success == true) 
    {
      Result = 1;   
    }
    
    return Result;
}




// Establish TCP/IP-connection to TCP/IP-Server
// returns 1 when ok, 0 when error

int TCPClient_Connect(String IPAddressString, int Port)
{
    int Result = 0;

    esp8266.setTimeout(5000);
    
    // send AT-command to ESP8266 to connect to TCP/IP-server
    esp8266.println("AT+CIPSTART=\"TCP\",\"" + IPAddressString + "\"," + String(Port) + "\r\n");

    // waits until ESP8266 send back "OK" (success = true) or "ERROR" (success = false)
    boolean success = esp8266.findUntil("OK", "ERROR");
    
    esp8266.setTimeout(1000);

    if (success == true) 
    {
      Result = 1;   
    }
    

    return Result;
}



// send out bytes via TCP/IP to server
// data contains the bytes to be sent out, len is the number of bytes to be sent out
// returns 1 when ok, 0 when error

int TCPClient_Send(byte data[], int len)
{
     int Result = 0;
     
     String Send = "AT+CIPSEND=" + String(len) + "\r\n";
     
     esp8266.println(Send);
     
     // waits until ESP8266 send back "OK" (success = true) or "ERROR" (success = false)
     boolean success = esp8266.findUntil("OK", "ERROR");

     if (success == true)
     {
        esp8266.write(data,len);

        // waits until ESP8266 send back "OK" (success = true) or "ERROR" (success = false)
        success = esp8266.findUntil("OK", "ERROR");
     }
     
     if (success == true) 
     {
       Result = 1;   
     }

     return Result;
}


// +++++++++++++++++++ End of TCP library +++++++++++++++++++++




String txtSSID = "YOUR SSID";
String txtPassword = "YOUR PASSWORD";


#define LED_BOARD  13
#define LED_GREEN  10
#define LED_RED    9

#define MODE_TEST      0
#define MODE_SENSOR    1

int DeviceMode = MODE_TEST;


boolean buttonState = LOW;
boolean oldbuttonState = LOW;
byte ButtonCount = 0;


void setup()
{
  
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BOARD, OUTPUT);
  
  pinMode(8, INPUT_PULLUP);

  boolean buttonState = LOW;
  boolean oldbuttonState = LOW;
  DeviceMode = MODE_TEST;


  
  // Open serial communications and wait for port to open:
  Serial.begin(19200);
  // set the data rate for the SoftwareSerial port
  esp8266.begin(19200);
  


  boolean success = true;
   
  // send AT-command to ESP8266 to reset the chip
  
  esp8266.setTimeout(5000);
  
  esp8266.println("AT+RST");

  // waits until ESP8266 sends back "OK" (success = true) or "ERROR" (success = false)
  success &= esp8266.findUntil("OK", "ERROR");

  
  // send AT-command to ESP8266 to put ESP8266 in client mode
  
  esp8266.setTimeout(1000);
  
  esp8266.println("AT+CWMODE=1");

  // waits until ESP8266 sends back "OK" (success = true) or "ERROR" (success = false)
  success &= esp8266.findUntil("OK", "ERROR");


  delay(2000);   // we need this delay to be sure that we react on the right "OK"

 
  
  // send AT-command to ESP8266 to log on local WiFi network
  
  esp8266.setTimeout(10000);
  
  esp8266.println ("AT+CWJAP=\"" + String(txtSSID) + "\",\"" + String(txtPassword) + "\"");

  // waits until ESP8266 sends back "OK" (success = true) or "ERROR" (success = false)
  success &= esp8266.findUntil("OK", "ERROR");

  
  esp8266.setTimeout(1000);
  
  if (success == true)
    {
       
      // green LED on
      digitalWrite(10, HIGH);
      
    }
    else
    {
      // red LED on
      digitalWrite(9, HIGH);
      
    }   

}


// converts number 0..15 in ASCII-code of Hex-Char
int HexAscii(int number)
{
  number = number & 0x0F;
  if (number < 10)
  {
    return 48 + number;    
  }
  else
  {
    return 64 + number - 9;
  }
}



void SendValueOverMQTT(int TenBitValue)
{
    // converts Value 0..1023 in 3 ASCII-coded hex-chars
    byte payload[3] = {0, 0, 0};
    
    payload[0] = HexAscii((TenBitValue & 0xF00) >> 8);
    payload[1] = HexAscii((TenBitValue & 0x0F0) >> 4);
    payload[2] = HexAscii(TenBitValue & 0x00F);
    
    // closes TCP connection
    TCPClient_Close();
    

    // connect to HiveMQ MQTT Testserver
    
    int TCP_Connect_Result = TCPClient_Connect("broker.hivemq.com", 1883);

    // Success?
    if (TCP_Connect_Result == 0)
    {
      // no = red LED
      digitalWrite(LED_RED, HIGH);
      digitalWrite(LED_GREEN, LOW);
    }
    else
    {
      // yes = green and red LED (yellow)
      digitalWrite(LED_GREEN, HIGH);
      digitalWrite(LED_RED, HIGH);

      // send out MQTT-Connect-Command
      MQTTClient_Connect("elektorIoT");
      delay(1000);

      // send out Publish-Command
      MQTTClient_Publish("/ElektorMyJourneyIoT/TestTopic/test", payload, 3);

      // only green LED
      digitalWrite(LED_RED, LOW);
    }

    delay(1000);  // to slow down the transmissions to test server a little bit
}



void loop() // run over and over
{
  
  // read state of button
  buttonState = digitalRead(8);

 
  // button was pressed
  if (buttonState == LOW && oldbuttonState == HIGH)
  {
    
    // toggle DeviceMode and LED on the board
    if (DeviceMode == MODE_SENSOR)
    {
      DeviceMode = MODE_TEST;
      digitalWrite(LED_BOARD, LOW);
    }
    else
    {
      DeviceMode = MODE_SENSOR;
      digitalWrite(LED_BOARD, HIGH);
    }
    
  }

  oldbuttonState = buttonState;


  
  if (DeviceMode == MODE_SENSOR)
  {
    // get ADC-Value at Pin A6  (10 bit)
    int light = analogRead(6);

    // send value via MQTT
    SendValueOverMQTT(light);
  }
  else
  {
    // Test mode
    // chars from PC via USB are getting to ESP8266 and vice versa
    if (esp8266.available())
    {
      Serial.write(esp8266.read());
    }
  
      
    if (Serial.available())
    {
      esp8266.write(Serial.read());  
    }
    
  }
  
  
}


